%****************************************************************************
%  MATLAB DATA PROCESSING CODE
%  
%  Version: DataProcessingFinal
%  Date: 09.10.2016
%  
%  Copyright 2016 by Fabian Tobias Mayer (fabianmayer@web.de)
%  This code may not be used for purposes which are not related to 
%  the master thesis of Fabian Tobias Mayer without his prior
%  written consent. 
%****************************************************************************

%----------------------------------------------------------------------------
clear;        % Remove variables from the work space
close all;    % Close all plots
clc;          % Clear the command window

%----------------------------------------------------------------------------
%% User Interface

%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
% Data acquisition:
% - Roller ski needs to be unloaded for the first second when the system
%   is started
%   (otherwise the tare function will lead to an error)
% - Data needs to be copied into the folder TestData
%   (all sub files of the measurement need to be copied,
%    only one measurement can be processed at a time)
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
% Direction of the coordinate axes:
% x-axis: Aligned with the beam of the roller ski
% y-axis: Horizontal, perpendicular to the beam of the roller ski
% z-axis: Vertical, perpendicular to the beam of the roller ski
 
% -----Position for which the loads should be calculated---------
% (offset to the neutral layers and the binding pin)
 
% Position of the neutral layer:
% - in x direction: The plane which intersects with the binding pin
% - in y direction: The plane in the geometric middle of the beam
% - in z direction: The plane which is 14.78 mm offset in negative
%                   z direction under the top of the beam of the roller 
%                   ski where the binding and the roller ski are
 
xS=0;               % Position in x direction [m]
yS=0;               % Position in y direction [m]
zS=0.01478;         % Position in z direction [m]
 
% !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%                       !!! NO CHANGES UNDERNEATH !!!
% !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%%-------------------------------Data Processing-----------------------------
 
% Function:
% 0) Parameters
%    0.1) Data Acquisition Parameters
%    0.2) Material Parameters
%    0.3) Geometric Parameters
%    0.4) Calibration Parameters
% 1) Import Data from SD Card
%    1.1) Import Data
%    1.2) Shift the date to start at 0ms and tare the measured values
% 2) Determine the Strain at the Strain Gauges
% 3) Calibration of the calculated strain output
% 4) Determine the Stress at the Rosette Positions with the
%    SkiPositionStress function
% 5) Calculation of the loads
%    5.1) Calculate them for the neutral layers
%    5.2) Calculate them for the selected position
% 6) Results
%    6.1) Sampling frequency
%    6.2) Gain state
%    6.3) Selected position
%    6.4) Max and min loads
% 7) Plots
%    7.1) Results
%    7.2) Plots
% 8) Write Results and Plots to Results
%    8.1) Generate new sub folder
%    8.2) Write the results as .xls-file
%    8.3) Safe the plots
%    8.4) Write command window output to .txt-file
%
% User interface:
% - Roller ski needs to be unloaded for the first second when the system
%   is started
%   (otherwise the tare function will lead to an error)
% - Data needs to be copied into the folder TestData
%   (all sub files of the measurement need to be copied, only one 
%   measurement can be processed at a time)

%%-------------------------------0) Parameters-------------------------------

% 0.1) Data Acquisition Parameters:

k=2.05;              % k-factor of the strain gauges
GL=333.33;           % Gain for low load setting
GH=223.08;           % Gain for high load setting
U0=3.3;              % Bridge excitation voltage
URange=3.3;          % Measuring range of the Arduino analogue input
res=4096;            % Possible values of the 12 bit resulution of the
                     % analogue input pin of the arduino
 
% 0.2) Material Parameters:

E=68.9e9;            % Young's modulus [Pa]
nu=0.33;             % Poisson's ratio [-]
 
% 0.3) Geometric Parameters:

A_S=0.00020521;      % Intersectional area of the profile [m^2]
h_min=1.5/1000;      % Minimal thickness of the profile (at the position 
                          % of the left and right strain gauges) [m]
I_y=2.139939429e-8;       % I_y second moment of inertia for the torque
% in y direction [m^4]
I_z=3.703968513e-8;       % I_z second moment of inertia for the torque
                          % in z direction [m^4]
d=180/1000;               % Horizontal distance from the strain gauges to the
                          % coordinate system [m]
z_T=(14.77979714-3.5)/1000;    % Distance of the top strain gauges to the
                               % coordinate system
% (in the centre of area of the profile) [m]
y_R=38/(2*1000);               % Distance of the right strain gauges to the
                               % coordinate system
                               % (in the centre of area of the profile) [m]
z_B=(28.5-14.77979714)/1000;   % Distance of the bottom strain gauges to the
                               % coordinate system
                               % (in the centre of area of the profile) [m]
y_L=38/(2*1000);               % Distance of the left strain gauges to the
                               % coordinate system
                               % (in the centre of area of the profile) [m]

% 0.4) Calibration Parameters:

%Strain Calibration:
cFrontTa=-1;
cFrontTb=-1;
cFrontTc=-1;
 
cFrontRa=-1;
cFrontRb=-1;
cFrontRc=-1;
 
cFrontBa=-1;
cFrontBb=-1;
cFrontBc=-0.9;
 
cFrontLa=-1;
cFrontLb=-1;
cFrontLc=-1;
 
cBackTa=-1;
cBackTb=-1;
cBackTc=-0.8;
 
cBackRa=-1;
cBackRb=-1;
cBackRc=-1;
 
cBackBa=2;
cBackBb=-1;
cBackBc=4.5;
 
cBackLa=-1;
cBackLb=-1;
cBackLc=-1;
 
% Load Calibration:
c_Fx=-0.2;
c_Fy=1;
c_Fz=1.87;
c_Mx=1;
c_My=1;
c_Mz=1;
 
%%------------------------------1) Import------------------------------------
 
% 1.1) Import Data:
 
% Store the names of the files
list=dir('TestData/*.txt');     % Determine the .txt-files in the folder
                                % and store them in the list
 
noOfFiles=length(list);         % Determine the number of sub files

measData=[];                    % Initialise matrix for the measured Data
 
% Write the measured data of the sub files under each other with increasing
% sub measuring no.
for i=1:noOfFiles
    %  fileName=list(i).name;                  % Get the filenames
    fileName=['TestData/',list(i).name];       % Define the filenames with
                                               % search path
    measData=[measData;dlmread(fileName)];     % Read the data from the file
                                               % and store it under the last
                                               % measurement
end
 
% 1.2) Shift Time and Tare:
 
% Time Shift:
% Shift the data that the first sample is at 0 ms
lengthData=size(measData,1);
measData(1:lengthData,1)=measData(1:lengthData,1)-measData(1,1);
 
% Tare:
% Tare system with the mean value of the first 20 samples
% (system needs to be unloaded)
for i=2:25
    if lengthData>=20
        meanTare=mean(measData(1:20,i));
        measData(1:lengthData,i)=measData(1:lengthData,i)-meanTare;
    else
        measData(1:lengthData,i)=measData(1:lengthData,i)-measData(1,i);
    end
end
 
%%---------------------------2) Strain Calculation---------------------------
 
% Define the matrix in which time and strain is stored
strainData=zeros(lengthData,25);
 
% Write the time in the first column
strainData(1:lengthData,1)=measData(1:lengthData,1);
 
% Determine which gain was set:
if measData(1,26)==1         % Low loads
    G=GL;
elseif measData(1,26)==2     % High loads
    G=GH;
end
% Calculate the strain and write it to the columns
for i=2:25
    strainData(1:lengthData,i)=4*URange/(k*U0*G*res)*measData(1:lengthData,i);
end
 
%%---------------------------3) Strain Calibration---------------------------
 
strainData(:,2)=cFrontTa*strainData(:,2);
strainData(:,3)=cFrontTb*strainData(:,3);
strainData(:,4)=cFrontTc*strainData(:,4);
strainData(:,5)=cFrontRa*strainData(:,5);
strainData(:,6)=cFrontRb*strainData(:,6);
strainData(:,7)=cFrontRc*strainData(:,7);
strainData(:,8)=cFrontBa*strainData(:,8);
strainData(:,9)=cFrontBb*strainData(:,9);
strainData(:,10)=cFrontBc*strainData(:,10);
strainData(:,11)=cFrontLa*strainData(:,11);
strainData(:,12)=cFrontLb*strainData(:,12);
strainData(:,13)=cFrontLc*strainData(:,13);
 
strainData(:,14)=cBackTa*strainData(:,14);
strainData(:,15)=cBackTb*strainData(:,15);
strainData(:,16)=cBackTc*strainData(:,16);
strainData(:,17)=cBackRa*strainData(:,17);
strainData(:,18)=cBackRb*strainData(:,18);
strainData(:,19)=cBackRc*strainData(:,19);
strainData(:,20)=cBackBa*strainData(:,20);
strainData(:,21)=cBackBb*strainData(:,21);
strainData(:,22)=cBackBc*strainData(:,22);
strainData(:,23)=cBackLa*strainData(:,23);
strainData(:,24)=cBackLb*strainData(:,24);
strainData(:,25)=cBackLc*strainData(:,25);
 
%%----------------------------4) Stress calculation--------------------------
 
% Extrackt the stress of the rosettes
strainRosetteFrontT=strainData(1:lengthData,2:4);
strainRosetteFrontR=strainData(1:lengthData,5:7);
strainRosetteFrontB=strainData(1:lengthData,8:10);
strainRosetteFrontL=strainData(1:lengthData,11:13);
strainRosetteBackT=strainData(1:lengthData,14:16);
strainRosetteBackR=strainData(1:lengthData,17:19);
strainRosetteBackB=strainData(1:lengthData,20:22);
strainRosetteBackL=strainData(1:lengthData,23:25);
 
% Use the SkiPositionStress function to determine the stresses at the
% positions of the rosettes:
 
% Orientation of the strain gauge rosettes:
%     1: Top, pins in positive x-direction
%     2: Top, pins in negative x-direction
%     3: Right, pins in positive z-direction
%     4: Bottom, pins in positive x-direction
%     5: Bottom, pins in negative x-direction
%     6: Left, pins in positive z-direction
 
%Rosette1 Front Top:
[sigma_x_FrontT,sigma_y_FrontT,sigma_z_FrontT,tau_xy_FrontT,tau_xz_FrontT]=SkiPositionStress(2,strainRosetteFrontT,E,nu);
%Rosette2 Front Right:
[sigma_x_FrontR,sigma_y_FrontR,sigma_z_FrontR,tau_xy_FrontR,tau_xz_FrontR]=SkiPositionStress(3,strainRosetteFrontR,E,nu);
%Rosette3 Front Bottom:
[sigma_x_FrontB,sigma_y_FrontB,sigma_z_FrontB,tau_xy_FrontB,tau_xz_FrontB]=SkiPositionStress(5,strainRosetteFrontB,E,nu);
%Rosette4 Front Left:
[sigma_x_FrontL,sigma_y_FrontL,sigma_z_FrontL,tau_xy_FrontL,tau_xz_FrontL]=SkiPositionStress(6,strainRosetteFrontL,E,nu);
 
%Rosette5 Back Top:
[sigma_x_BackT,sigma_y_BackT,sigma_z_BackT,tau_xy_BackT,tau_xz_BackT]=SkiPositionStress(1,strainRosetteBackT,E,nu);
%Rosette6 Back Right:
[sigma_x_BackR,sigma_y_BackR,sigma_z_BackR,tau_xy_BackR,tau_xz_BackR]=SkiPositionStress(3,strainRosetteBackR,E,nu);
%Rosette7 Back Bottom:
[sigma_x_BackB,sigma_y_BackB,sigma_z_BackB,tau_xy_BackB,tau_xz_BackB]=SkiPositionStress(4,strainRosetteBackB,E,nu);
%Rosette8 Back Left:
[sigma_x_BackL,sigma_y_BackL,sigma_z_BackL,tau_xy_BackL,tau_xz_BackL]=SkiPositionStress(6,strainRosetteBackL,E,nu);
 
%%-----------------------------5) Load Calculation---------------------------
 
% 5.1) Calculate the loads from the measured strain:

%      - The forces and torques are calculated for the position at the
%        centre of area in the middle between the front and back sensors
%        (is in the middle of the beam under the binding link)
 
% F_x=-F_xFront+F_xBack:
F_x=c_Fx*A_S*((sigma_x_BackR+sigma_x_BackL)/2-(sigma_x_FrontR+sigma_x_FrontL)/2);

% F_y (compare bottom and top to eliminate normal forces
% and add front and back because forces have the same effect in the front
% and in the back):
F_y=c_Fy*I_z/(2*d)*((sigma_x_BackL/y_L-sigma_x_BackR/y_R)+(sigma_x_FrontL/y_L-sigma_x_FrontR/y_R));

% F_z (same proceeding like for F_y):
F_z=c_Fz*I_y/(2*d)*((sigma_x_FrontT/z_T-sigma_x_FrontB/z_B)+(sigma_x_BackT/z_T-sigma_x_BackB/z_B));
 
% M_x=-M_xFront+M_xBack:
M_x=c_Mx*A_S*h_min/2*((tau_xz_BackL-tau_xz_BackR)-(tau_xz_FrontL-tau_xz_FrontR));

% M_y (compare bottom and top to eliminate normal forces and subtract front 
% and back because torques have different effects in the front
% and in the back):
M_y=c_My*I_y/2*((sigma_x_BackT/z_T-sigma_x_BackB/z_B)-(sigma_x_FrontT/z_T-sigma_x_FrontB/z_B));

% M_z (same proceeding like for M_y):
M_z=-c_Mz*I_z/2*((sigma_x_BackL/y_L-sigma_x_BackR/y_R)-(sigma_x_FrontL/y_L-sigma_x_FrontR/y_R));
 
% 5.2) Calculate the loads for the selected position

F_xS=F_x;
F_yS=F_y;
F_zS=F_z;
M_xS=M_x-F_y*zS+F_z*yS;
M_yS=M_y+F_x*zS-F_z*xS;
M_zS=M_z-F_x*yS+F_y*xS;
 
%%----------------------------------6) Results-------------------------------

% 8.1) Generate a new sub folder in the results folder

formatOut = 'yyyy/mm/dd/HH/MM/SS';
timeNow=datestr(now,formatOut);
timeNowNew=strrep(timeNow,'/','_');
folderDirect=['Results\',timeNowNew];
mkdir(folderDirect);
 
% 8.4) Write Command Window output to a .txt-file

folderDirectResultsTXT=[folderDirect,'\1_Results.txt'];
diary (folderDirectResultsTXT);
 
disp('--------------------------------------------------------------------');
disp('---------------------------RESULTS----------------------------------');
disp('--------------------------------------------------------------------');
 
% Extract the time vector for further calculations and the plots
time=strainData(1:lengthData,1);
 
% 6.1) Sampling frequency:

disp('SAMPLING FREQUENCY');
% Calculate the time increment between two measures
timeInc=zeros(lengthData,1);    % Define the vector to store the time inc.
timeInc(1,1)=0;                 % First increment is zero
 
for i=1:lengthData-1            % Calculate the other increments
    timeInc(i,1)=time(i+1,1)-time(i,1);
end
 
fMax=1000/min(timeInc);           % Maximal sampling frequency [Hz]
fMean=1000/mean(timeInc);         % Mean sampling frequency [Hz]
fMin=1000/max(timeInc);           % Minimal sampling frequency [Hz]
 
% Return the frequencies on the command window:
disp(['Mean sampling frequency [Hz]: ',num2str(fMean)]);
disp(['Lowest sampling frequency [Hz]: ',num2str(fMin)]);
 
disp('--------------------------------------------------------------------');
 
% 6.2) Show whether the gain was adapted for high or low forces

disp('AMPLIFICATION');
if measData(1,26)==1
    disp('Gain was adapted set for: LOW LOADS (limited measuring range)');
elseif measData(1,26)==2
    disp('Gain was adapted set for: HIGH LOADS (lower accuracy');
end
 
disp('--------------------------------------------------------------------');
 
% 6.3) Position for which the loads were calculated

disp('POSITION');
disp(['Position for which the loads are calculated (x,y,z) [m]: ',num2str(xS),' , ',num2str(yS),' , ',num2str(zS)]);
 
disp('--------------------------------------------------------------------');

% 6.4) Maximum loads

% Determine the maximum and minimum values of the forces and torques and
% when they happened
[F_xSMin,F_xSMinI]=min(F_xS);
[F_ySMin,F_ySMinI]=min(F_yS);
[F_zSMin,F_zSMinI]=min(F_zS);
[M_xSMin,M_xSMinI]=min(M_xS);
[M_ySMin,M_ySMinI]=min(M_yS);
[M_zSMin,M_zSMinI]=min(M_zS);
 
[F_xSMax,F_xSMaxI]=max(F_xS);
[F_ySMax,F_ySMaxI]=max(F_yS);
[F_zSMax,F_zSMaxI]=max(F_zS);
[M_xSMax,M_xSMaxI]=max(M_xS);
[M_ySMax,M_ySMaxI]=max(M_yS);
[M_zSMax,M_zSMaxI]=max(M_zS);
 
% Output them on the display
disp('LOADS');
disp(['F_x: minimum ',num2str(F_xSMin),' (at ',num2str(time(F_xSMinI,1)/1000),' sec) | maximum ',num2str(F_xSMax),' (at ',num2str(time(F_xSMaxI,1)/1000),' sec)']);
disp(['F_y: minimum ',num2str(F_ySMin),' (at ',num2str(time(F_ySMinI,1)/1000),' sec) | maximum ',num2str(F_ySMax),' (at ',num2str(time(F_ySMaxI,1)/1000),' sec)']);
disp(['F_z: minimum ',num2str(F_zSMin),' (at ',num2str(time(F_zSMinI,1)/1000),' sec) | maximum ',num2str(F_zSMax),' (at ',num2str(time(F_zSMaxI,1)/1000),' sec)']);
disp(['M_x: minimum ',num2str(M_xSMin),' (at ',num2str(time(M_xSMinI,1)/1000),' sec) | maximum ',num2str(M_xSMax),' (at ',num2str(time(M_xSMaxI,1)/1000),' sec)']);
disp(['M_y: minimum ',num2str(M_ySMin),' (at ',num2str(time(M_ySMinI,1)/1000),' sec) | maximum ',num2str(M_ySMax),' (at ',num2str(time(M_ySMaxI,1)/1000),' sec)']);
disp(['M_z: minimum ',num2str(M_zSMin),' (at ',num2str(time(M_zSMinI,1)/1000),' sec) | maximum ',num2str(M_zSMax),' (at ',num2str(time(M_zSMaxI,1)/1000),' sec)']);
 
disp('--------------------------------------------------------------------');
 
diary off;
 
%%-------------------------------7) Plots------------------------------------

% Plot of the loads F_xS, F_yS, F_zS for the selected position
figure;
hold on
plot(time,F_xS,'r:','Linewidth',1);
plot(time,F_yS,'g--','Linewidth',1);
plot(time,F_zS,'b-','Linewidth',1);
grid on;
set(gca,'FontWeight','normal','FontSize',12,'FontName','Palatino Linotype');
xlabel('Time [ms]')
ylabel('Force [N]')
legend('{\itF_x}','{\itF_y}','{\itF_z}');
title('FORCES')
 
% Plot of the loads M_xS, M_yS, M_zS for the selected position
figure;
hold on
plot(time,M_xS,'r:','Linewidth',1);
plot(time,M_yS,'g--','Linewidth',1);
plot(time,M_zS,'b-','Linewidth',1);
grid on;
set(gca,'FontWeight','normal','FontSize',12,'FontName','Palatino Linotype');
xlabel('Time [ms]')
ylabel('Torque [Nm]')
legend('{\itM_x}','{\itM_y}','{\itM_z}');
title('TORQUES')
 
%%-------------------------8) Safe the Results and Plots---------------------
 
% 8.2) Write measured data to an excel-file:

folderDirectXls=[folderDirect,'\2_Load Logfile'];
outputmatrix=[time,F_x,F_y,F_z,M_x,M_y,M_z];
header = {'Time [ms]' 'F_x [N]' 'F_y [N]' 'F_z [N]' 'M_x [Nm]' 'M_y [Nm]' 'M_z [Nm]'};
xlswrite(folderDirectXls, header, 'sheetname'); % by defualt starts from A1
xlswrite(folderDirectXls, outputmatrix , 'sheetname','A2'); % array under the header.
 
% 8.3) Safe the figures to the folder:

folderDirectForcesXls=[folderDirect,'\3_Forces.fig'];
folderDirectTorqueXls=[folderDirect,'\4_Torques.fig'];
saveas(figure (1),folderDirectForcesXls);
saveas(figure (2),folderDirectTorqueXls);

